/*
 * TimeLimit Copyright <C> 2019 - 2022 Jonas Lochmann
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */
package io.timelimit.android.data.model

import androidx.room.*
import io.timelimit.android.extensions.base64
import io.timelimit.android.extensions.toByteArray
import java.security.MessageDigest

@Entity(
    tableName = "user_u2f_key",
    foreignKeys = [
        ForeignKey(
            entity = User::class,
            parentColumns = ["id"],
            childColumns = ["user_id"],
            onUpdate = ForeignKey.CASCADE,
            onDelete = ForeignKey.CASCADE
        )
    ],
    indices = [
        Index(
            value = ["key_handle", "public_key"],
            unique = true
        )
    ]
)
data class UserU2FKey (
    @PrimaryKey(autoGenerate = true)
    @ColumnInfo(name = "key_id")
    val keyId: Long,
    @ColumnInfo(name = "user_id", index = true)
    val userId: String,
    @ColumnInfo(name = "added_at")
    val addedAt: Long,
    @ColumnInfo(name = "key_handle")
    val keyHandle: ByteArray,
    @ColumnInfo(name = "public_key")
    val publicKey: ByteArray,
    @ColumnInfo(name = "next_counter")
    val nextCounter: Long
) {
    fun calculateServerKeyIdSync() = MessageDigest.getInstance("SHA256").also {
        it.update(keyHandle.size.toByteArray())
        it.update(keyHandle)

        it.update(publicKey.size.toByteArray())
        it.update(publicKey)
    }.digest()
        .sliceArray(0 until 6)
        .base64()
}